
import uuid
from datetime import datetime
from freezegun import freeze_time
from unittest.mock import patch
from dateutil.relativedelta import relativedelta
from odoo.exceptions import AccessError
from odoo.addons.saas_worker import metabase
from odoo.addons.saas_worker.auth_util import tokenize_param
from odoo.addons.saas_trial.tests.common import TestTrialCommon, tagged
from odoo.addons.saas_worker.util import version_gte


@tagged('post_install', '-at_install', 'post_saas_trial_creation')
class TestTrialSelfActivation(TestTrialCommon):

    def test_resend_activation_backend(self):
        # force database as to be activated
        metabase.set_expiry_in(self.env.cr.dbname, '4 hours', set_status='to_activate')
        with patch('requests.post') as request_post_mock:
            request_post_mock.return_value.json.return_value = {'result': {'confirm_token': tokenize_param('test-confirm-token')}}
            self.start_tour('/web', 'trial_resend_activation_email_backend', login=self.user_admin.login)
            request_post_mock.assert_called_once()


@tagged('post_install', '-at_install', 'post_saas_trial_creation')
class TestTrialSelfVarious(TestTrialCommon):

    def test_cron_inalterability(self):
        """Ensure that no user than modifiy/archive update notification and vacuum cron."""
        notif_cron = self.env.ref('mail.ir_cron_module_update_notification')
        with self.assertRaises(AccessError, msg="no user should be able to modify update notifcation cron"):
            notif_cron.sudo().active = False
        with self.assertRaises(AccessError, msg="no user should be able to delete update notification cron"):
            notif_cron.sudo().unlink()

        vacuum_cron = self.env.ref('base.autovacuum_job')
        with self.assertRaises(AccessError, msg="no user should be able to modify vacuum cron"):
            vacuum_cron.sudo().active = False
        with self.assertRaises(AccessError, msg="no user should be able to delete vacuum cron"):
            vacuum_cron.sudo().unlink()

    def test_dbuuid_inalterability(self):
        new_uuid = str(uuid.uuid4())
        admin_env = self.env(user=self.user_admin)  # compat. for v12.0, with_user() only exists in v13.0+
        with self.assertRaises(AccessError, msg="users should be able to change the database uuid"):
            admin_env["ir.config_parameter"].set_param('database.uuid', new_uuid)
        with self.assertRaises(AccessError, msg="users should be able to remove the database uuid"):
            admin_env["ir.config_parameter"].set_param('database.uuid', None)

    def test_demo_install_grace_period(self):
        # test grace period: only allowing demo install for 4 hours after the database creation
        with freeze_time(datetime.now() + relativedelta(hours=4, minutes=1)):
            with self.assertRaises(AccessError):
                self.env["ir.demo"].install_demo()

    def test_my_subscriptions_menu(self):

        if not version_gte("15.0"):
            self.skipTest("The 'My Subscriptions' menu was only introduced for v15.0+")
        self.start_tour('/web', 'trial_my_subscriptions_menuitem', login=self.user_admin.login)
